hdbase   equ  1f0h      ; 0x1f0 for primary device
                         ; 0x170 for secondary device
                         ;
hdid     equ  0h       ; 0x00 for master hd
                         ; 0x10 for slave hd
; Port 64

fMouse struct
	x dword 0
	y dword 0
	mb dword 0
	rb dword 0
	lb dword 0
fMouse ends

WAIT_OUTPUT equ 0
WAIT_WRITE equ 1
WAIT_COMMAND equ 2
WAIT_READ equ 3
WAIT_READ60 equ 4

mouse_cycle dd 0
mbyte1 byte 0
mbyte2 byte 0
mbyte3 byte 0
mouse fMouse <0>
.code

;Mouse


PortWait proc a_type:dword
	
;A read from port 64H gives the following status byte:
;
;   Bit     Function
;    7      1 = Parity error
;    6      1 = General Time Out
;    5      1 = Auxiliary output buffer full
;    4      1 = Inhibit switch
;    3      1 = Command/data
;    2      1 = System flag
;    1      1 = Input buffer full
;    0      1 = Output buffer full	
	cli
	.if a_type==WAIT_READ
		wait_type_0:
		in al,64h
		bt ax,6
		jc error
		bt ax,7
		jc error
		bt ax,WAIT_READ			; if bit 3 set data is available at 64h
		jnc wait_type_0
		xor eax,eax
	.elseif a_type==WAIT_WRITE
		wait_type_1:			; Bit 1 should be zero before we able to send command
		in al,64h
		bt ax,6
		jc error
		bt ax,7
		jc error
		bt ax,WAIT_WRITE
		jc wait_type_1
		xor eax,eax
	.elseif a_type==WAIT_COMMAND
		wait_type_2:			; Bit 0 and 1 should be zero before we able to send command
		in al,64h
		bt ax,6
		jc error
		bt ax,7
		jc error
		bt ax,WAIT_COMMAND
		jc wait_type_2
		xor eax,eax
	.elseif a_type==WAIT_READ60
		wait_type_4:
		in al,64h
		bt ax,6
		jc error
		bt ax,7
		jc error
		bt ax,WAIT_READ			; if bit 3 not set data is available at 60h
		jc wait_type_4
		xor eax,eax
	.elseif a_type==WAIT_OUTPUT
		wait_type_5:
		in al,64h
		bt ax,6
		jc error
		bt ax,7
		jc error
		bt ax,WAIT_OUTPUT			; if bit 3 set data is available at 64h
		jnc wait_type_5
		xor eax,eax
	.endif
	
	error:
	sti
	ret
PortWait endp

PortRead proc
	
	invoke PortWait,WAIT_READ
	in al,60h
	
	ret
PortRead endp

PortWrite proc a_cmd:byte
	
	invoke PortWait,WAIT_WRITE
	mov al,a_cmd
	out 64h,al
	
	ret
PortWrite endp

KeyBoardRead proc
	LOCAL key:byte
	
	invoke PortWrite,0AEh
	invoke PortWrite,0D2h
	in al,60h
;	invoke PortRead
;	mov key,al
;	invoke PortWrite,0AEh
;	mov al,key
;	push eax
;		push eax
;		invoke dw2bin,eax,base_buffer
;		invoke Print,base_buffer
;		pop eax
;		invoke dw2hex,eax,base_buffer
;		invoke Print,base_buffer
;		invoke PrintRel,addr crg
;	pop eax
;	invoke PortWrite,0A7h
	ret
KeyBoardRead endp

MouseWaitACK proc
	
	_wait:
	in al,60h
	cmp al,0FAh
	jnz _wait
	
	ret
MouseWaitACK endp

MouseSendCommand proc cmd:byte
	
	invoke PortWrite,0D4h
	mov al,cmd
	out 60h,al
	invoke MouseWaitACK
	
	ret
MouseSendCommand endp

MouseInit proc
	LOCAL mID:byte
	LOCAL packet[4]:byte
	;Get Mouse ID
	; Reset
	invoke StoreMem,addr mouse
	mov esi,eax
	invoke memfill,eax,sizeof mouse,0
	
	invoke PortWrite,0A8h
	invoke MouseSendCommand,0FFh
	invoke MouseSendCommand,0F6h
	invoke MouseSendCommand,0F4h
	invoke MouseSendCommand,0F0h
	
	invoke MouseSendCommand,0F2h
	in al,60h
	mov mID,al

	invoke MouseSendCommand,0F3h
	invoke MouseSendCommand,0Ah
	invoke MouseSendCommand,0F2h
	in al,60h
	.if al!=0
		mov mID,al		; Had Track Ball
	.endif
	
	;	Activate Wheel
	invoke MouseSendCommand,0F3h
	invoke MouseSendCommand,0C8h
	invoke MouseSendCommand,0F3h
	invoke MouseSendCommand,064h
	invoke MouseSendCommand,0F3h
	invoke MouseSendCommand,050h
	in al,60h
	.if al==3
		mov mID,al
	.endif
	
	mov al,mID
	ret
MouseInit endp

MouseRead proc uses esi
	LOCAL packet[4]:byte
	LOCAL status[4]:byte
	
	
	cli
	invoke MouseSendCommand,0F4h
	invoke MouseSendCommand,0E9h
	in al,60h
	mov status[0],al
	in al,60h
	mov status[1],al
	in al,60h
	mov status[2],al
	in al,60h
	mov status[3],al
	
	invoke MouseSendCommand,0EBh
	in al,60h
	mov packet[0],al
	in al,60h
	mov packet[1],al
	in al,60h
	mov packet[2],al
	in al,60h
	mov packet[3],al
	
	lea ecx,status
	mov ecx,[ecx]
	bswap ecx
	lea edx,packet
	mov edx,[edx]
	bswap edx
	
	invoke StoreMem,addr mouse
	mov esi,eax
	
	
	movzx eax,packet[2]
	bt ax,7
	jc y_minus
	add [esi].fMouse.y,eax
	mov eax,vga_info1
	movzx eax,[eax].ModeInfoBlock.YResolution
	.if [esi].fMouse.y>eax
		mov eax,vga_info1
		movzx eax,[eax].ModeInfoBlock.YResolution
		mov [esi].fMouse.y,eax
	.endif
	jmp done_y
	y_minus:
		not al
		inc eax
		sub [esi].fMouse.y,eax
		.if SIGN?
			mov [esi].fMouse.y,0
		.endif
	done_y:
	
	
	movzx eax,packet[1]
	bt ax,7
	jc x_minus
	add [esi].fMouse.x,eax
	mov eax,vga_info1
	movzx eax,[eax].ModeInfoBlock.XResolution
	.if [esi].fMouse.x>eax
		mov eax,vga_info1
		movzx eax,[eax].ModeInfoBlock.XResolution
		mov [esi].fMouse.x,eax
	.endif
	jmp done_x
	x_minus:
		not al
		inc eax
		sub [esi].fMouse.x,eax
		.if SIGN?
			mov [esi].fMouse.x,0
		.endif
	done_x:
	
	mov al,packet[0]
	mov [esi].fMouse.mb,eax
	
	lea eax,packet
	mov eax,[eax]
	bswap eax
	movzx eax,packet[1]
	push eax
		push eax
		invoke StoreMem,addr mouse
		mov esi,eax
		invoke dw2a,[esi].fMouse.x,base_buffer
		invoke Print,base_buffer
		invoke PrintRel,CADD(" ")
		pop eax
		invoke StoreMem,addr mouse
		mov esi,eax
		invoke dw2a,[esi].fMouse.y,base_buffer
		invoke Print,base_buffer
		invoke PrintRel,addr crg
	pop eax
	invoke MouseSendCommand,0F5h
	sti
	
	ret
MouseRead endp



;DISK
ReadSector proc nBlock:dword,lpbuffer:dword

    pushad 
    mov eax, nBlock
    push  eax
	
  newhdread:
   
    mov   edx,hdbase
    inc   edx
    mov   al,0
    out   dx,al

    inc   edx
    mov   al,1
    out   dx,al

    inc   edx
    pop   ax
    out   dx,al

    inc   edx
    shr   ax,8
    out   dx,al

    inc   edx
    pop   ax
    out   dx,al

    inc   edx
    shr   ax,8
    and   al,1+2+4+8
    add   al,hdid
    add   al,128+64+32
    out   dx,al

    inc   edx
    mov   al,20h
    out   dx,al
   
  hdwait:
   
    in    al,dx
    test  al,128
    jnz   hdwait

    mov   edi,lpbuffer 
    mov   ecx,256
    mov   edx,hdbase
    cld
    rep   insw

    popad

    ret
ReadSector Endp

;keyboard
pWaitKey proc
	LOCAL o60:byte
	local o61:byte
	LOCAL keyp:byte
	
	mov keyp,0
	
	mov dx,60h
	init_keyboard:
	in al,dx
	bt ax,7
	jc no_keyboard
	jmp loop_wait
	
	no_keyboard:
	mov o60,al
	inc dx
	in al,dx
	mov o61,al
	and al,011111111b
	out dx,al
	in al,dx
	cmp al,o61
	jz error
	
	loop_wait:
	mov dx,60h
	in al,dx
	cmp al,0
	jz loop_wait
	mov keyp,al
	error:
	
	restore_state:
	mov dx,61h
	mov al,o61
	out dx,al
	dec dx
	mov al,o60
	out dx,al
	
	movzx eax,keyp
	
	ret
pWaitKey endp

; Loaded 16-bit Driver